(* Code interface.
	This interface defines the types from which the program tree
	is constructed before being saved to disk.
*)

(*
	05/12/96	Created.
*)

INTERFACE Code;

IMPORT Tags;

	(* The Version record is used to stop modules loading in programs
		created by more advanced interpreters. *)
TYPE	Version = REF RECORD
			ver := Tags.Version;
			rev := Tags.Revision;
		END;

	(* The Node type is used to represent commands, functions, etc. *)
	(* The "extra" field will usually be NIL, unless the node represents
		a variable or constant, in which case it may provide other info.
		The type of a node is determined by a Tag.T code.
	*)
	Node = RECORD
			type	: Tags.T	:= Tags.T.NO_OP;
			params	: ParamR	:= NIL;
			next	: T		:= NIL;
			extra	: REFANY	:= NIL;
		END;
	T = REF Node;

	(* Parameters to commands/functions are stored in a Param array
		linked to the Node.
	*)
	Param = RECORD
			next	: ParamR	:= NIL;
			param	: T		:= NIL;
		END;
	ParamR = REF Param;

	(* Some storage elements for constants. *)
	BoolR = REF RECORD value : BOOLEAN END;
	RealR = REF RECORD value : LONGREAL END;
	IntR = REF RECORD value : INTEGER END;
	(* Note that strings are already refs, so we don't worry about them. *)

END Code.	