(* Lexer interface.
	The lexer module provides some basic functions for
	getting tokens out of text files.
*)

(*
	05/12/96	Created
	09/12/96	Removed the "mustbe" method because it never gets used.
	12/12/96	Added a new "eof" method so that the parser can tell
			whether or not we've parsed everything.
	02/02/97	Added a new "ip_address" method to scan for an IP address.
			For convenience, I have said that any character except
			white-space may be in an IP address.
	19/02/97	Added a new "fromtext" method to allow a lexer object to
			scan a text string rather than a file.
*)

INTERFACE Lexer;

IMPORT OSError;

	(* BadToken is raised by any lexer operation that
		encounters text invalid for that token type. *)
EXCEPTION BadToken;

TYPE	T <: Public;
	Public = OBJECT
		METHODS
			init(filename : TEXT) : T RAISES {OSError.E};
			fromtext(t : TEXT) : T;

			expect(token : TEXT) : BOOLEAN;

			ident() : TEXT RAISES {BadToken};
			real() : LONGREAL RAISES {BadToken};
			integer() : INTEGER RAISES {BadToken};
			string() : TEXT RAISES {BadToken};
			ip_address() : TEXT RAISES {BadToken};

			nextbytes() : TEXT;

			eof() : BOOLEAN;
			close();
		END;

END Lexer.