(* Main module for Armadillo 3D Terminal program. *)

(*
	25/11/96	Created
	01/12/96	Close method implemented for network
			display object.
	
*)

MODULE Main;

(* Modula-3 libs: *)
IMPORT IO, Params, Text;

(* Custom 3D graphics window routine. *)
IMPORT Window;

VAR display : Window.T;
	dispname : TEXT := NIL;
	dispmode := Window.ModeSet{Window.Mode.Colour};

(* Procedure to cope with special command line tokens. *)
PROCEDURE Special(token : TEXT)=
BEGIN

	IF Text.Equal(token, "/SOLID") THEN
		dispmode := dispmode + Window.ModeSet{Window.Mode.Solid};
	ELSIF Text.Equal(token, "/WIRE") THEN
		dispmode := dispmode - Window.ModeSet{Window.Mode.Solid};
	ELSIF Text.Equal(token, "/MONO") THEN
		dispmode := dispmode - Window.ModeSet{Window.Mode.Colour};
	ELSIF Text.Equal(token, "/COLOUR") THEN
		dispmode := dispmode + Window.ModeSet{Window.Mode.Colour};
	ELSIF Text.Equal(token, "/?") THEN
		Usage();
	ELSE
		IO.Put("I don't understand the tag " & token & "\n");
		Usage();
	END;

END Special;

(* Procedure to print Usage information. *)
PROCEDURE Usage()=
BEGIN

	IO.Put("Usage : " & Params.Get(0) & " <display-name> [/SOLID] [/WIRE] [/MONO] [/COLOUR]\n");
	IO.Put("This terminal supports colour.\n");

END Usage;

BEGIN

	IO.Put("\nArmadillo 3D Terminal version 0.3\n");

	(* Examine the parameter line for interesting stuff. *)
	IF Params.Count > 1 THEN
		FOR x:=1 TO Params.Count-1 DO
			IF Text.GetChar(Params.Get(x), 0) = '/' THEN

				(* This is a command-line parameter, so do
					something special with it. *)
				Special(Params.Get(x));

			ELSE

				(* Have we already got a terminal name? *)
				IF dispname = NIL THEN
					(* No, so use this one. *)
					dispname := Params.Get(x);
				ELSE
					(* Yes, so print an error. *)
					dispname := NIL;
					EXIT;
				END;	
			END;
		END;
	END;

	IF dispname = NIL THEN
		Usage();
	ELSE

		TRY
			(* Open the output window. *)
			display := NEW(Window.T).init(dispname, dispmode);

			(* And wait for it to be deleted. *)
			IO.Put("Close output window to quit.\n");
			display.awaitDelete();
		EXCEPT
			Window.Failed=>
				IO.Put("Could not open output window.\n");
		|	Window.NetFailed=>
				IO.Put("Could not open networking.\n");
		END;
	END;

END Main.