(* AtomExp module.
	Implements some useful functions and stuff.
*)

(*
	18/12/96	Created Skeleton.
	23/12/96	Added an object for the "str" function.
	13/01/96	SEE NODE MODULE.
	24/01/97	Added an object for the "time" value.
	19/02/97	Fixed the random function to cope with -ve values.
*)

MODULE AtomExp;

IMPORT Random, Time AS TimeSys;

IMPORT Node, Loader;
IMPORT Number, Strings, Bool;

IMPORT Code, Tags;

VAR	random := NEW(Random.Default).init();

(* Bracketted sub-expression. *)
REVEAL	Bracket = Node.T BRANDED OBJECT
			expr : Node.T;
		OVERRIDES
			get_value := Bracket_GetValue;
			print := Bracket_Print;
		END;

PROCEDURE Bracket_GetValue(self : Bracket; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN self.expr.get_value(env);
END Bracket_GetValue;

PROCEDURE Bracket_Print(self : Bracket; i : TEXT) : TEXT=
BEGIN
	RETURN "(" & self.expr.print(i) & ")";
END Bracket_Print;

(* Boolean negation. *)
REVEAL	Not = Node.T BRANDED OBJECT
			expr : Node.T;
		OVERRIDES
			get_value := Not_GetValue;
			print := Not_Print;
		END;

PROCEDURE Not_GetValue(self : Not; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Bool.NewBool(NOT(self.expr.get_value(env).get_bool(env)));
END Not_GetValue;

PROCEDURE Not_Print(self : Not; i : TEXT) : TEXT=
BEGIN
	RETURN "not " & self.expr.print(i);
END Not_Print;

(* Random number generator. *)
REVEAL	Rnd = Node.T BRANDED OBJECT
			expr : Node.T;
		OVERRIDES
			get_value := Rnd_GetValue;
			print := Rnd_Print;
		END;

PROCEDURE Rnd_GetValue(self : Rnd; env : Node.E) : Node.V RAISES {Node.Abort}=
VAR	range : LONGREAL;
BEGIN
	range := self.expr.get_value(env).get_real(env);
	IF range < 0.0d0 THEN
		RETURN Number.NewNumber(random.longreal(range, 0.0d0));
	ELSE
		RETURN Number.NewNumber(random.longreal(0.0d0, range));
	END;
END Rnd_GetValue;

PROCEDURE Rnd_Print(self : Rnd; i : TEXT) : TEXT=
BEGIN
	RETURN "rnd " & self.expr.print(i);
END Rnd_Print;

(* Datatype to string converter. *)
REVEAL	Str = Node.T BRANDED OBJECT
			expr : Node.T;
		OVERRIDES
			get_value := Str_GetValue;
			print := Str_Print;
		END;

PROCEDURE Str_GetValue(self : Str; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Strings.NewString(self.expr.get_value(env).printval(env));
END Str_GetValue;

PROCEDURE Str_Print(self : Str; i : TEXT) : TEXT=
BEGIN
	RETURN "str " & self.expr.print(i);
END Str_Print;

(* Monadic plus. *)
REVEAL	Plus = Node.T BRANDED OBJECT
			expr : Node.T;
		OVERRIDES
			get_value := Plus_GetValue;
			print := Plus_Print;
		END;

PROCEDURE Plus_GetValue(self : Plus; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN self.expr.get_value(env);
END Plus_GetValue;

PROCEDURE Plus_Print(self : Plus; i : TEXT) : TEXT=
BEGIN
	RETURN "+" & self.expr.print(i);
END Plus_Print;

(* Monadic minus. *)
REVEAL	Minus = Node.T BRANDED OBJECT
			expr : Node.T;
		OVERRIDES
			get_value := Minus_GetValue;
			print := Minus_Print;
		END;

PROCEDURE Minus_GetValue(self : Minus; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Number.NewNumber(-(self.expr.get_value(env).get_real(env)));
END Minus_GetValue;

PROCEDURE Minus_Print(self : Minus; i : TEXT) : TEXT=
BEGIN
	RETURN "-" & self.expr.print(i);
END Minus_Print;

(* Random number generator. *)
REVEAL	Time = Node.T BRANDED OBJECT
		OVERRIDES
			get_value := Time_GetValue;
			print := Time_Print;
		END;

PROCEDURE Time_GetValue(self : Time; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Number.NewNumber(TimeSys.Now());
END Time_GetValue;

PROCEDURE Time_Print(self : Time; i : TEXT) : TEXT=
BEGIN
	RETURN "time";
END Time_Print;

(* Routine to convert Code trees into the above. *)
PROCEDURE Convert(tree : Code.T) : Node.T RAISES {Loader.Failure}=
BEGIN

	IF tree = NIL THEN RETURN NIL; END;

	CASE tree.type OF
		Tags.T.EXP_Bracket=>
			RETURN NEW(Bracket,
				expr := Loader.Convert(tree.params.param)
				);
	|	Tags.T.BOOL_Not=>
			RETURN NEW(Not,
				expr := Loader.Convert(tree.params.param)
				);
	|	Tags.T.FUN_Rnd=>
			RETURN NEW(Rnd,
				expr := Loader.Convert(tree.params.param)
				);
	|	Tags.T.FUN_Str=>
			RETURN NEW(Str,
				expr := Loader.Convert(tree.params.param)
				);
	|	Tags.T.MONO_Plus=>
			RETURN NEW(Plus,
				expr := Loader.Convert(tree.params.param)
				);
	|	Tags.T.MONO_Minus=>
			RETURN NEW(Minus,
				expr := Loader.Convert(tree.params.param)
				);
	|	Tags.T.TAG_Time=>
			RETURN NEW(Time);
	ELSE
		RETURN NIL;
	END;

END Convert;

BEGIN
END AtomExp.