(* Bool module.
	Implements the bool data type.
*)

(*
	15/12/96	Created.
	10/01/97	Added get_typecode methods to value and constructor
			objects.
	13/01/97	SEE NODE MODULE.
	19/02/97	Implemented boolean equality test.
*)

MODULE Bool;

IMPORT Code, Tags;

IMPORT Node, TypeSys;

(* The basic bool constant type. *)
REVEAL	Bool = Node.K BRANDED OBJECT
			value : BOOLEAN;
		OVERRIDES
			get_bool := Bool_GetBool;
			print := Bool_Print;
			printval := Bool_PrintVal;
			equal := Bool_Equal;
		END;

PROCEDURE Bool_GetBool(self : Bool; <* UNUSED *> env : Node.E) : BOOLEAN=
BEGIN
	RETURN self.value;
END Bool_GetBool;

PROCEDURE Bool_Print(self : Bool; <* UNUSED *> i : TEXT) : TEXT=
BEGIN
	IF self.value THEN
		RETURN "true";
	ELSE
		RETURN "false";
	END;
END Bool_Print;

PROCEDURE Bool_PrintVal(self : Bool; <* UNUSED *> env : Node.E) : TEXT=
BEGIN
	RETURN self.print("");
END Bool_PrintVal;

PROCEDURE Bool_Equal(self : Bool; env : Node.E; val : Node.K) : BOOLEAN RAISES {Node.Abort}=
BEGIN
	RETURN self.value = val.get_bool(env);
END Bool_Equal;

(* Sub-type which also allows assignment. *)
REVEAL	BoolV = Bool BRANDED OBJECT
		OVERRIDES
			set_value := BoolV_SetValue;
		END;

PROCEDURE BoolV_SetValue(self : BoolV; env : Node.E; val : Node.K) RAISES {Node.Abort}=
BEGIN
	self.value := val.get_bool(env);
END BoolV_SetValue;

(* Allocator object, representing the type. *)
REVEAL	BoolT = Node.TYP BRANDED OBJECT
		OVERRIDES
			allocate := BoolT_Allocate;
			check_type := BoolT_CheckType;
			print := BoolT_Print;
		END;

PROCEDURE BoolT_Allocate(<* UNUSED *> self : BoolT; <* UNUSED *> env : Node.E) : Node.K=
BEGIN
	RETURN NEW(BoolV, value := FALSE);
END BoolT_Allocate;

PROCEDURE BoolT_CheckType(<* UNUSED *> self : BoolT; <* UNUSED *> env : Node.E; <* UNUSED *> i : Node.InvalidTypes) : BOOLEAN=
BEGIN
	(* Booleans cannot be recursive. *)
	RETURN TRUE;
END BoolT_CheckType;

PROCEDURE BoolT_Print(<* UNUSED *> self : BoolT; <* UNUSED *> i : TEXT) : TEXT=
BEGIN
	RETURN "bool";
END BoolT_Print; 

(* Routine to produce the above objects. *)
PROCEDURE Convert(tree : Code.T) : Node.T=
BEGIN

	IF tree = NIL THEN RETURN NIL; END;

	CASE tree.type OF
		Tags.T.TAG_Boolean=>
			RETURN NEW(Node.V).init(
					BoolType,
					NEW(Bool, value := NARROW(tree.extra, Code.BoolR).value)
					);
	|	Tags.T.TYP_Boolean=>
			RETURN BoolType;
	ELSE
		RETURN NIL;
	END;

END Convert;

(* Routine for producing new booleans of set value. *)
PROCEDURE NewBool(val : BOOLEAN) : Node.V=
BEGIN
	RETURN NEW(Node.V).init(BoolType, NEW(BoolV, value := val));
END NewBool;

BEGIN
	BoolType := NEW(BoolT, code := TypeSys.NewTypeCode());
END Bool.