(* Except interface.
	This interface defines all the Armadillo system
	exceptions and will eventually provide a list of
	them for the NewBaseEnv function to use.
*)

(*
	12/12/96	Created.
	13/12/96	Added "unsupported" exception.
	14/12/96	Added "untrapped" exception.
	17/12/96	Added exceptions for the run-time limiting system.
	19/12/96	Added the "nosuchfield" exception for records.
			Added "derefnil" for references.
	20/12/96	Added "threadreturnval" for thread handling.
	11/01/97	Added "unlockedvar" and "alreadylocked" exceptions
			for use in locking.
	14/01/97	Added "ioerror" exception to deal with IO library
			failures sensibly.
	25/01/97	Added the "infinitetype" exception to deal with infinite types.
	19/02/97	Added the "quit" exception for terminating execution.
*)

INTERFACE Except;

IMPORT Node;

CONST	NoSuchExcept = "no_such_exception";
	NoSuchVar = "no_such_var";
	NoSuchType = "no_such_type";

	BadType = "type_mismatch";
	InfiniteType = "infinite_type";
	DuplicateType = "duplicate_type";

	NoTurtle = "no_turtle";

	(* From the Graphics module. *)
	NoDisplay = "no_display";
	Alerted = "alerted";
	TurtleUsed = "turtle_used";

	(* A few language constructs don't work in this interpreter. *)
	Unsupported = "unsupported";

	(* For the "loop" construct. *)
	Exit = "exit";

	(* For the exception commands. *)
	Untrapped = "untrapped";

	(* For function calling. *)
	NumParams = "numparam";
	Return = "return";

	(* For ArmRT limiting routines. *)
	TooManyThreads = "too_many_threads";
	TooManyFunctions = "too_many_functions";

	(* For records. *)
	NoSuchField = "no_such_field";

	(* For references. *)
	DerefNil = "deref_nil";

	(* For threads. *)
	ThreadReturnVal = "thread_return_val";

	(* For locking. *)
	UnlockedVar = "unlocked_var";
	AlreadyLocked = "already_locked";

	(* For IO. *)
	IOError = "io_error";

	(* Quitting run-time. *)
	Quit = "quit";

PROCEDURE System() : Node.ExceptList;

END Except.