(* Except module.
	This module provides the System() function used by
	Node.NewBaseEnv for setting up the initial set of
	system exceptions.
*)

(*
	12/12/96	Created.
	13/12/96	Added the "unsupported" exception, to cope
			with language constructs that the compiler can
			handle, but which the interpreter can't.
	16/12/96	Add the "return" exception for function calls.
	19/12/96	Added the "nosuchfield" exception for records.
			Added "derefnil" for references.
	14/01/97	Added the "ioerror" exception.
	25/01/97	Added the "infinitetype" exception to cope with infinite types.
	19/02/97	Added the "quit" exception to deal with termination.
*)

MODULE Except;

IMPORT Node;

TYPE T=Node.ExceptList;

(* Procedure which gathers all the exceptions in the interface and
	puts them in an ExceptList. *)
PROCEDURE System() : T=
VAR	a : T;
BEGIN

	(* The most important exception.  Everything dies without it! *)
	a := NEW(T).init(NoSuchExcept);

	a := NEW(T).init(NoSuchVar).setnext(a);
	a := NEW(T).init(BadType).setnext(a);
	a := NEW(T).init(InfiniteType).setnext(a);
	a := NEW(T).init(DuplicateType).setnext(a);
	a := NEW(T).init(NoTurtle).setnext(a);
	a := NEW(T).init(NoDisplay).setnext(a);
	a := NEW(T).init(Alerted).setnext(a);
	a := NEW(T).init(TurtleUsed).setnext(a);
	a := NEW(T).init(Unsupported).setnext(a);
	a := NEW(T).init(Exit).setnext(a);
	a := NEW(T).init(Untrapped).setnext(a);
	a := NEW(T).init(NumParams).setnext(a);
	a := NEW(T).init(Return).setnext(a);
	a := NEW(T).init(TooManyThreads).setnext(a);
	a := NEW(T).init(TooManyFunctions).setnext(a);
	a := NEW(T).init(NoSuchField).setnext(a);
	a := NEW(T).init(DerefNil).setnext(a);
	a := NEW(T).init(NoSuchType).setnext(a);
	a := NEW(T).init(ThreadReturnVal).setnext(a);
	a := NEW(T).init(UnlockedVar).setnext(a);
	a := NEW(T).init(AlreadyLocked).setnext(a);
	a := NEW(T).init(IOError).setnext(a);
	a := NEW(T).init(Quit).setnext(a);

	RETURN a;

END System;

BEGIN
END Except.