(* GfxTurtle interface.
	Defines the graphics turtle type.  This type uses the armturtle
	library as it's base and adds actual display functionality to
	it.
*)

(*
	12/12/96	Created.
	12/01/97	Added the set_colour method, to allow coloured line
			drawing to be available.
*)

INTERFACE GfxTurtle;

IMPORT Turtle;

IMPORT Display;

EXCEPTION	NoDisplay;
		NoDisconnect;
		CannotLock;

TYPE	T <: Public;
	Public = Turtle.T OBJECT
		METHODS

			(* Attach a turtle to a display. *)
			attach(display : Display.T) RAISES {NoDisplay, NoDisconnect};

			(* Set the drawing mode - on or off. *)
			pendown(flag : BOOLEAN);

			(* The following are methods implemented by the Turtle.T type.
				The GfxTurtle.T adds the graphical functionality to
				these methods, based on the original methods.
			*)
			forward(dist : Turtle.Units) RAISES {NoDisplay};
			backward(dist : Turtle.Units) RAISES {NoDisplay};

			(* To set the pen colour. *)
			set_colour(r, g, b : REAL);

			(* Routines for locking turtles. *)
			lock_turtle() RAISES {CannotLock};
			unlock_turtle();

			(* Duplication routine. *)
			duplicate(new : T) : T;

			(* Clear graphics command. *)
			clear() RAISES {NoDisplay};

		END;

PROCEDURE New() : T;

PROCEDURE Cleanup();

PROCEDURE ClearAll();

END GfxTurtle.