(* Strings module.
	Implements the string data type.
*)

(*
	15/12/96	Created.
	10/01/97	Added the get_typecode method.
	14/01/97	MAJOR RE-WRITE.  SEE NODE MODULE.
*)

MODULE Strings;

IMPORT Text;

IMPORT Code, Tags, TypeSys;

IMPORT Node;

(* The basic string constant type. *)
REVEAL	String = Node.K BRANDED OBJECT
			value : TEXT;
		OVERRIDES
			get_string := String_GetString;
			print := String_Print;
			printval := String_PrintVal;
			compare := String_Compare;
		END;

PROCEDURE String_GetString(self : String; <* UNUSED *> env : Node.E) : TEXT=
BEGIN
	RETURN self.value;
END String_GetString;

PROCEDURE String_Print(self : String; <* UNUSED *> i : TEXT) : TEXT=
BEGIN
	RETURN "\"" & self.value & "\"";
END String_Print;

PROCEDURE String_PrintVal(self : String; <* UNUSED *> env : Node.E) : TEXT=
BEGIN
	RETURN self.value;
END String_PrintVal;

PROCEDURE String_Compare(self : String; env : Node.E; val : Node.K) : Node.Result RAISES {Node.Abort}=
BEGIN
	RETURN Text.Compare(self.value, val.get_string(env));
END String_Compare;

(* Sub-type which also allows assignment. *)
REVEAL	StringV = String BRANDED OBJECT
		OVERRIDES
			set_value := StringV_SetValue;
		END;

PROCEDURE StringV_SetValue(self : StringV; env : Node.E; val : Node.K) RAISES {Node.Abort}=
BEGIN
	self.value := val.get_string(env);
END StringV_SetValue;

(* Allocator object, representing the type. *)
REVEAL	StringT = Node.TYP BRANDED OBJECT
		OVERRIDES
			allocate := StringT_Allocate;
			check_type := StringT_CheckType;
			print := StringT_Print;
		END;

PROCEDURE StringT_Allocate(<* UNUSED *> self : StringT; <* UNUSED *> env : Node.E) : Node.K=
BEGIN
	RETURN NEW(StringV, value := "");
END StringT_Allocate;

PROCEDURE StringT_CheckType(<* UNUSED *> s : StringT; <* UNUSED *> env : Node.E; <* UNUSED *> i : Node.InvalidTypes) : BOOLEAN=
BEGIN
	(* Strings cannot be recursive. *)
	RETURN TRUE;
END StringT_CheckType;

PROCEDURE StringT_Print(<* UNUSED *> self : StringT; <* UNUSED *> i : TEXT) : TEXT=
BEGIN
	RETURN "string";
END StringT_Print; 

(* Routine to produce the above objects. *)
PROCEDURE Convert(tree : Code.T) : Node.T=
BEGIN

	IF tree = NIL THEN RETURN NIL; END;

	CASE tree.type OF
		Tags.T.TAG_String=>
			RETURN NEW(Node.V).init(
					StringType,
					NEW(String, value := tree.extra)
					);
	|	Tags.T.TYP_String=>
			RETURN StringType;
	ELSE
		RETURN NIL;
	END;

END Convert;

(* Routine to allocate an instantiated string. *)
PROCEDURE NewString(value : TEXT) : Node.V=
BEGIN
	RETURN NEW(Node.V).init(StringType, NEW(String, value := value));
END NewString;

BEGIN
	StringType := NEW(StringT, code := TypeSys.NewTypeCode());
END Strings.