(* Term module.
	Implements objects for basic expression
	constructs.
*)

(*
	15/12/96	Created.
			Still need PrintVal methods implementing.
	14/01/97	MAJOR RE-WRITE. SEE NODE MODULE.
	18/01/97	Added the integer division and remainder functions.
*)

MODULE Term;

IMPORT Node, Loader;
IMPORT Number, Bool;

IMPORT Code, Tags;

(* Multiplication object. *)
REVEAL	Mult = Node.T BRANDED OBJECT
			left, right : Node.T;
		OVERRIDES
			get_value := Mult_GetValue;
			print := Mult_Print;
		END;

PROCEDURE Mult_GetValue(self : Mult; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Number.NewNumber(
		self.left.get_value(env).get_real(env) *
		self.right.get_value(env).get_real(env)
		);
END Mult_GetValue;

PROCEDURE Mult_Print(self : Mult; i : TEXT) : TEXT=
BEGIN
	RETURN self.left.print(i) & " * " & self.right.print(i);
END Mult_Print;

(* Division object. *)
REVEAL	Div = Node.T BRANDED OBJECT
			left, right : Node.T;
		OVERRIDES
			get_value := Div_GetValue;
			print := Div_Print;
		END;

PROCEDURE Div_GetValue(self : Div; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Number.NewNumber(
		self.left.get_value(env).get_real(env) /
		self.right.get_value(env).get_real(env)
		);
END Div_GetValue;

PROCEDURE Div_Print(self : Div; i : TEXT) : TEXT=
BEGIN
	RETURN self.left.print(i) & " / " & self.right.print(i);
END Div_Print;

(* Logical AND. *)
REVEAL	And = Node.T BRANDED OBJECT
			left, right : Node.T;
		OVERRIDES
			get_value := And_GetValue;
			print := And_Print;
		END;

PROCEDURE And_GetValue(self : And; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Bool.NewBool(
		self.left.get_value(env).get_bool(env) AND
		self.right.get_value(env).get_bool(env)
		);
END And_GetValue;

PROCEDURE And_Print(self : And; i : TEXT) : TEXT=
BEGIN
	RETURN self.left.print(i) & " and " & self.right.print(i);
END And_Print;

(* Integer division object. *)
REVEAL	IntDiv = Node.T BRANDED OBJECT
			left, right : Node.T;
		OVERRIDES
			get_value := IntDiv_GetValue;
			print := IntDiv_Print;
		END;

PROCEDURE IntDiv_GetValue(self : IntDiv; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Number.NewNumber(
		FLOAT(
			TRUNC(self.left.get_value(env).get_real(env)) DIV
			TRUNC(self.right.get_value(env).get_real(env)),
			LONGREAL)
		);
END IntDiv_GetValue;

PROCEDURE IntDiv_Print(self : IntDiv; i : TEXT) : TEXT=
BEGIN
	RETURN self.left.print(i) & " div " & self.right.print(i);
END IntDiv_Print;

(* Division object. *)
REVEAL	IntMod = Node.T BRANDED OBJECT
			left, right : Node.T;
		OVERRIDES
			get_value := IntMod_GetValue;
			print := IntMod_Print;
		END;

PROCEDURE IntMod_GetValue(self : IntMod; env : Node.E) : Node.V RAISES {Node.Abort}=
BEGIN
	RETURN Number.NewNumber(
		FLOAT(
			TRUNC(self.left.get_value(env).get_real(env)) MOD
			TRUNC(self.right.get_value(env).get_real(env)),
			LONGREAL)
		);
END IntMod_GetValue;

PROCEDURE IntMod_Print(self : IntMod; i : TEXT) : TEXT=
BEGIN
	RETURN self.left.print(i) & " mod " & self.right.print(i);
END IntMod_Print;

(* Routine to convert Code trees into the above. *)
PROCEDURE Convert(tree : Code.T) : Node.T RAISES {Loader.Failure}=
BEGIN

	IF tree = NIL THEN RETURN NIL; END;

	CASE tree.type OF
		Tags.T.EXP_Times=>
			RETURN NEW(Mult,
				left := Loader.Convert(tree.params.param),
				right := Loader.Convert(tree.params.next.param)
				);
	|	Tags.T.EXP_Divide=>
			RETURN NEW(Div,
				left := Loader.Convert(tree.params.param),
				right := Loader.Convert(tree.params.next.param)
				);
	|	Tags.T.BOOL_And=>
			RETURN NEW(And,
				left := Loader.Convert(tree.params.param),
				right := Loader.Convert(tree.params.next.param)
				);
	|	Tags.T.EXP_IntDiv=>
			RETURN NEW(IntDiv,
				left := Loader.Convert(tree.params.param),
				right := Loader.Convert(tree.params.next.param)
				);
	|	Tags.T.EXP_IntMod=>
			RETURN NEW(IntMod,
				left := Loader.Convert(tree.params.param),
				right := Loader.Convert(tree.params.next.param)
				);
	ELSE
		RETURN NIL;
	END;

END Convert;

BEGIN
END Term.